@@ -188,7 +188,7 @@ import hmac
 import re
 import json
 import requests
-from datetime import datetime
+from datetime import datetime, timedelta
 
 import boto3
 import os
@@ -616,7 +616,11 @@ class Auth(object):
         self.id_token = id_token
         self.access_token = access_token
         self.refresh_token = refresh_token
-        self.expires = datetime.fromtimestamp(id_verify.get("exp"))
+        self.expires = datetime.fromtimestamp(
+            id_verify.get("exp")
+        ) - timedelta(
+            minutes=5
+        )  ## prevent just about to expire tokens from being used
 
     def get_tokens(self):
         if not self.id_token:
